package tourplaner.business;

import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.layout.*;
import javafx.scene.text.Text;
import javafx.stage.Stage;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.time.LocalDate;
import java.util.Date;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicReference;

public class AlertHelper {

    /**
     * Warning Dialog
     * @param title Title des Dialogs
     * @param header Header des Dialogs
     * @param msg Nachricht des Dialogs
     */
    public static void warn(String title, String header, String msg){
        alertType(Alert.AlertType.WARNING, title, header, msg);
    }

    /**
     * Info Dialog
     * @param title Title des Dialogs
     * @param header Header des Dialogs
     * @param msg Nachricht des Dialogs
     */
    public static void inform(String title, String header, String msg){
        alertType(Alert.AlertType.INFORMATION, title, header, msg);
    }

    /**
     * Info Dialog ohne Header
     * @param title Title des Dialogs
     * @param msg Nachricht des Dialogs
     */
    public static void informNoHeader(String title, String msg){
        inform(title, null, msg);
    }

    /**
     * Error Dialog
     * @param title Title des Dialogs
     * @param header Header des Dialogs
     * @param msg Nachricht des Dialogs
     */
    public static void error(String title, String header, String msg){
        alertType(Alert.AlertType.ERROR, title, header, msg);
    }

    /**
     * Ausgabe einer Exception in einem Dialog
     * @param title Title des Dialogs
     * @param header Header des Dialogs
     * @param msg Nachricht des Dialogs
     * @param ex Die exception die ausgegeben werden soll
     */
    public static void exerror(String title, String header, String msg, Exception ex){
        Alert alert = new Alert(Alert.AlertType.ERROR);
        alert.setTitle(title);
        alert.setHeaderText(header);
        alert.setContentText(msg);
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        ex.printStackTrace(pw);
        String exceptionText = sw.toString();
        Label label = new Label(ConfigHelper.getLangIniString("exceptionstackheader"));
        TextArea textArea = new TextArea(exceptionText);
        textArea.setEditable(false);
        textArea.setWrapText(true);
        textArea.setMaxWidth(Double.MAX_VALUE);
        textArea.setMaxHeight(Double.MAX_VALUE);
        GridPane.setVgrow(textArea, Priority.ALWAYS);
        GridPane.setHgrow(textArea, Priority.ALWAYS);
        GridPane expContent = new GridPane();
        expContent.setMaxWidth(Double.MAX_VALUE);
        expContent.add(label, 0, 0);
        expContent.add(textArea, 0, 1);
        alert.getDialogPane().setExpandableContent(expContent);
        alert.showAndWait();
    }

    /**
     *
     * @param alertly Typ des Dialogs
     * @param title Title des Dialogs
     * @param header Header des Dialogs
     * @param msg Nachricht des Dialogs
     */
    private static void alertType(Alert.AlertType alertly, String title, String header, String msg){
        Alert alert = new Alert(alertly);
        alert.setTitle(title);
        alert.setHeaderText(header);
        alert.setContentText(msg);

        alert.showAndWait();
    }

    /**
     * Texteingabe
     * @param title Title des Dialogs
     * @param header Header des Dialogs
     * @param msg Nachricht des Dialogs
     * @return Null bei keiner eingabe
     */
    public static String inputText(String title, String header, String msg) {
        TextInputDialog dialog = new TextInputDialog();
        dialog.setTitle(title);
        dialog.setHeaderText(header);
        dialog.setContentText(msg);
        Optional<String> result = dialog.showAndWait();
        AtomicReference<String> returnText = new AtomicReference<>("");
        result.ifPresent(returnText::set);
        if(!returnText.get().isEmpty()){
            return returnText.get();
        }else{
            return null;
        }
    }

    /**
     * Positive Nummer eingabe. Wenn Convertierung zu int nicht klappt -> -1
     * @param title Title des Dialogs
     * @param header Header des Dialogs
     * @param msg Nachricht des Dialogs
     * @return -1 bei error sonst ein Int
     */
    public static int inputNumber(String title, String header, String msg) {
        TextInputDialog dialog = new TextInputDialog();
        dialog.setTitle(title);
        dialog.setHeaderText(header);
        dialog.setContentText(msg);
        Optional<String> result = dialog.showAndWait();
        AtomicReference<Integer> returnText = new AtomicReference<Integer>(-1);
        result.ifPresent(s -> {
            try {
                int resultInt = Integer.parseInt(result.get());
                returnText.set(resultInt);
            }catch (NumberFormatException e){
                AlertHelper.warn(ConfigHelper.getLangIniString("achtung"), "Bitte nur Zahlen eingeben!", "Bitte geben sie nur Zahlen von 0 - 10 an!");
                returnText.set(-1);
            }
        });

        return returnText.get();
    }

    /**
     * Date Picker Dialog, sobald ein Datum ausgewählt wurde, wird es automatisch bestätigt und der dialog geschlossen
     * @param title Name des Dialogs
     * @return Gewähltes datum
     */
    public static LocalDate datePicker(String title){
        final DatePicker datePicker = new DatePicker(LocalDate.now());
        final Stage stage = new Stage();
        AtomicReference<LocalDate> selectedDate = new AtomicReference<>();
        datePicker.setOnAction(event -> {
            LocalDate date = datePicker.getValue();
            System.out.println("Selected date: " + date);
            selectedDate.set(date);
            stage.close();
        });

        stage.setScene(
                new Scene(datePicker)
        );
        stage.showAndWait();
        return selectedDate.get();
    }
}
