package tourplaner.ui;

import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.geometry.Orientation;
import javafx.scene.control.*;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.MouseEvent;
import tourplaner.business.AlertHelper;
import tourplaner.business.ConfigHelper;
import tourplaner.business.LogHelper;
import tourplaner.object.Log;
import tourplaner.object.Tour;
import tourplaner.viewmodels.ViewModel;

import java.awt.*;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.time.LocalDate;
import java.util.ResourceBundle;
import java.util.UUID;
import java.util.concurrent.atomic.AtomicReference;

public class TourplanerController implements Initializable {
    //VM
    public ViewModel viewModel = new ViewModel();
    //Tour list -> links
    public ListView<String> TourListView = new ListView<>();
    //Tabs zu Tour -> rechts oben
    public TabPane viewTabPane;
    public Tab kartenTab, beschreibungTab;
    public TableView<Tour> beschreibungTableView;
    public TableColumn<Tour, String> startCol, zielCol, dauerCol, streckeCol, nameCol;
    public TextField titleTextView, sucheInput;
    //Log -> rechts unten
    public TableView<Log> logTableView;
    public TableColumn<Log, String> logDauerCol, logStreckeCol, logDatumCol;

    /**
     * Öffnet github im standart browser
     *
     */
    @FXML
    private void gitWebBrowser(){
       openBrowser(ConfigHelper.getIniString(ConfigHelper.getStandartConfig(), "hilfe", "gitrepo"), "Git");
    }

    /**
     * Öffnet Java Doc im standart browser
     *
     */
    @FXML
    private void javaDocBrowser(){
       openBrowser(ConfigHelper.getIniString(ConfigHelper.getStandartConfig(), "hilfe", "javadoc"), "JavaDoc");
    }

    /**
     * Öffnet Doxygen Doc im standart browser
     *
     */
    @FXML
    private void doxygenDocBrowser(){
        openBrowser(ConfigHelper.getIniString(ConfigHelper.getStandartConfig(), "hilfe", "doxygendoc"), "Doxygen");
    }

    /**
     * Öffnet einen link im Standart Browser
     * @param uriString
     * @param appname
     */
    private void openBrowser(String uriString, String appname){
        Desktop desktop = java.awt.Desktop.getDesktop();
        try {
            URI oURL = new URI(
                    uriString);
            desktop.browse(oURL);
        } catch (URISyntaxException | IOException e) {
            LogHelper.error(e.getMessage(), ConfigHelper.getLangIniString("openbrowsererror") + appname);
            AlertHelper.exerror(ConfigHelper.getLangIniString("browserexception"),
                    ConfigHelper.getLangIniString("openbrowsererror") + appname,
                    ConfigHelper.getLangIniString("browserexceptionmsg"), e);
        }
    }

    /**
     * Wird gestartet wenn eine Tour in der Tour listView ausgewählt wird
     * @param mouseEvent Triggered Event
     */
    @FXML
    private void tourListSelectedItem(MouseEvent mouseEvent){
        String selectedItem = TourListView.getSelectionModel().getSelectedItem();
        this.viewModel.selectTour(selectedItem);
        titleTextView.setText(selectedItem);
        beschreibungTableView.getItems().removeIf(s -> true); //Leert die Table View komplett
        beschreibungTableView.getItems().add(this.viewModel.getTour(selectedItem));
        startCol.setCellValueFactory(new PropertyValueFactory<Tour, String>("start"));
        zielCol.setCellValueFactory(new PropertyValueFactory<Tour, String>("ziel"));
        dauerCol.setCellValueFactory(new PropertyValueFactory<Tour, String>("dauer"));
        streckeCol.setCellValueFactory(new PropertyValueFactory<Tour, String>("strecke"));
        nameCol.setCellValueFactory(new PropertyValueFactory<Tour, String>("name"));

        //Log anzeigen
        logTableView.setPlaceholder(new Label( ConfigHelper.getLangIniString("keinelogsvorhanden")));
        logTableView.setItems(this.viewModel.getLogData());
        logDauerCol.setCellValueFactory(new PropertyValueFactory<Log, String>("dauer"));
        logStreckeCol.setCellValueFactory(new PropertyValueFactory<Log, String>("strecke"));
        logDatumCol.setCellValueFactory(new PropertyValueFactory<Log, String>("datum"));
    }

    /**
     * Beendet die App
     * Verbunden mit dem Menu -> Datei -> Beenden
     */
    @FXML
    private void quitApp(){
        System.exit(0);
    }

    /**
     * Fügt eine Tour hinzu
     * Verbunden mit Button -> Tour -> +
     */
    @FXML
    private void addTour(){
        this.viewModel.addTour();
    }

    /**
     * Entfernt eine ausgewählte Tour
     * Verbunden mit Button -> Tour -> -
     */
    @FXML
    private void delTour(){
        this.beschreibungTableView.getItems().removeIf(s -> true); //löscht alles aus der tabelle
        this.titleTextView.setText(ConfigHelper.getLangIniString("keinetourselected"));
        this.viewModel.delTour();
        this.viewModel.setSelectedTour(null);
        logTableView.setPlaceholder(new Label( ConfigHelper.getLangIniString("keinetourselected")));
    }

    /**
     * Sucht eine Tour
     * Verbunden mit Button -> Suche
     */
    @FXML
    private void suche(){
        String sucheInput = this.sucheInput.getText();
        if(sucheInput.isEmpty()){
            AlertHelper.warn(ConfigHelper.getLangIniString("achtung"),
                    ConfigHelper.getLangIniString("suchfeldleer"),
                    ConfigHelper.getLangIniString("suchtextzuerst"));
        }else {
            this.viewModel.suche(sucheInput);
        }
    }

    /**
     * Fügt einen Log eintrag zu einer Tour hinzu.
     * Ist keine Tour ausgewählt, dann kommt eine Warnung an den User!
     */
    @FXML
    private void addLog(){
        Tour selectedTour = this.viewModel.getSelectedTour();
        if (selectedTour == null){
            AlertHelper.warn(ConfigHelper.getLangIniString("achtung"),
                    ConfigHelper.getLangIniString("keinetourselected"),
                    ConfigHelper.getLangIniString("logtournotselectedmsg"));
        }else {
            ObservableList<Tour> tourData = this.viewModel.getTourData();
            tourData.forEach(s -> {
                if (s.getName().equals(selectedTour.getName())) {
                    ObservableList<Log> logData = this.viewModel.getLogData();
                    AtomicReference<String> newId = new AtomicReference<>();
                    newId.set(UUID.randomUUID().toString());
                    logData.forEach(ss -> {
                        if (ss.getId().equals(newId.get())) {
                            newId.set(UUID.randomUUID().toString());
                        }
                    });
                    double neueDauer = -1;
                    double neueStrecke = -1;
                    LocalDate neuesDatum = null;
                    while (neuesDatum == null) {
                        neuesDatum = AlertHelper.datePicker(ConfigHelper.getLangIniString("datum"));
                    }
                    while (neueDauer == -1) {
                        neueDauer = AlertHelper.inputNumber(ConfigHelper.getLangIniString("dauer"),
                                ConfigHelper.getLangIniString("dauermsg"),
                                ConfigHelper.getLangIniString("dauer") +
                                        ConfigHelper.getLangIniString("doppelpunkt"));
                    }
                    while (neueStrecke == -1) {
                        neueStrecke = AlertHelper.inputNumber(ConfigHelper.getLangIniString("strecke"),
                                ConfigHelper.getLangIniString("streckemsg"),
                                ConfigHelper.getLangIniString("strecke") +
                                        ConfigHelper.getLangIniString("doppelpunkt"));
                    }

                    Log newLog = new Log(newId.get(), neueDauer + "", neuesDatum, neueStrecke);
                    logData.add(newLog);
                    s.addLog(newLog);
                }
            });
        }
    }

    @FXML
    private void delLog(){
        Log selectedLog = this.viewModel.getSelectedLog();
        if(selectedLog != null) {
            this.viewModel.delLog(selectedLog.getId());
        }else{
            AlertHelper.warn(ConfigHelper.getLangIniString("achtung"),
                    ConfigHelper.getLangIniString("keinetourselected"),
                    ConfigHelper.getLangIniString("deltournoselectmsg"));
        }
    }

    /**
     * Ein Log eintrag wurde ausgewählt
     * @param mouseEvent Aktuelles Mouse Event
     */
    @FXML
    private void logItemSelected(MouseEvent mouseEvent){
        String selectedItem = logTableView.getSelectionModel().getSelectedItem().getId();
        this.viewModel.selectLog(selectedItem);
        System.out.println(selectedItem);
    }

    /**
     * Wird beim Start ausgeführt
     * @param url
     * @param resourceBundle
     */
    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        //Tour list -> links
        TourListView.getSelectionModel().setSelectionMode(SelectionMode.SINGLE);
        TourListView.setOrientation(Orientation.VERTICAL);
        TourListView.setItems(this.viewModel.getTourNamen());
        //Tabs zu Tour -> rechts oben
        beschreibungTableView.setPlaceholder(new Label( ConfigHelper.getLangIniString("keinetourselected")));
        titleTextView.setText( ConfigHelper.getLangIniString("keinetourselected"));
        //Log -> rechts unten
        logTableView.setPlaceholder(new Label( ConfigHelper.getLangIniString("keinetourselected")));

    }

    /**
     * Funktion für noch nicht implementierte sachen wie im Menu der 'Bearbeiten' und 'Optionen' Knopf
     */
    @FXML
    public void nimpButton(){
        AlertHelper.inform(ConfigHelper.getLangIniString("achtung"),
                ConfigHelper.getLangIniString("fktnichtimplementiert"),
                ConfigHelper.getLangIniString("vergessenodernochnichtsoweit"));
    }

}
