package tourplaner.viewmodels;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import tourplaner.business.AlertHelper;
import tourplaner.business.ConfigHelper;
import tourplaner.business.LogHelper;
import tourplaner.object.Log;
import tourplaner.object.Tour;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;

public class ViewModel {
    //Tour
    private final ObservableList<Tour> tourData = FXCollections.observableArrayList(new Tour("Test 1", "120", "json dings", 22.3, "Wien", "Graz"),new Tour("Test 2", "210", "json dings", 42.3, "Da", "Dort"));
    private final ObservableList<String> tourNamen = FXCollections.observableArrayList("Test 1", "Test 2");
    private Tour selectedTour;
    private String neueTourName, neueTourStart, neueTourZiel;
    //Log
    private final ObservableList<Log> logData = FXCollections.observableArrayList();
    private Log selectedLog;


    /**
     * Fügt eine neue Tour hinzu
     */
    public void addTour(){
        while(this.neueTourName == null) {
            this.neueTourName = AlertHelper.inputText(ConfigHelper.getLangIniString("tournametitle"),
                    ConfigHelper.getLangIniString("tournameheader"),
                    ConfigHelper.getLangIniString("tournamemsg"));
            if (getTour(this.neueTourName) != null) {
                AlertHelper.warn(ConfigHelper.getLangIniString("achtung"),
                        ConfigHelper.getLangIniString("namevergebenheader"),
                        ConfigHelper.getLangIniString("namevergebenmsg1")
                                + this.neueTourName +
                                ConfigHelper.getLangIniString("namevergebenmsg2"));
                this.neueTourName = null;
            }
        }
        while(this.neueTourStart == null){
            this.neueTourStart = AlertHelper.inputText(ConfigHelper.getLangIniString("startpunkttitle"),
                    ConfigHelper.getLangIniString("startpunktheader"),
                    ConfigHelper.getLangIniString("startpunktmsg"));
        }
        while(this.neueTourZiel == null){
            this.neueTourZiel = AlertHelper.inputText(ConfigHelper.getLangIniString("zielpunkttitle"),
                    ConfigHelper.getLangIniString("zielpunktheader"),
                    ConfigHelper.getLangIniString("zielpunktmsg"));
        }
        if (getTour(this.neueTourName) == null) {
            tourData.add(new Tour(this.neueTourName, "TBD", "TBD", 0, this.neueTourStart, this.neueTourZiel));
            tourNamen.add(this.neueTourName);
        }
        this.neueTourStart = null;
        this.neueTourZiel = null;
        this.neueTourName = null;
    }

    /**
     * Entfernt ein Log anhand dessen Id
     * @param id Id des Logs welches entfernt werden soll
     */
    public void delLog(String id){
        this.logData.removeIf(s -> s.getId().equals(id));
        AtomicReference<Tour> tourToEdit = new AtomicReference<>();
        this.tourData.forEach(s -> {
            if (s.getLogs() != null) {
                s.getLogs().forEach(ss -> {
                    if (ss.getId().equals(id)) {
                        tourToEdit.set(s);
                    }
                });
            }
        });

        if (tourToEdit.get() != null){
            Tour toEdit = tourToEdit.get();
            toEdit.delLog(id);
            this.tourData.removeIf(s -> s.getName().equals(toEdit.getName()));
            this.tourData.add(toEdit);
        }
    }

    /**
     * Selectiert ein Log anhand der Id
     * @param id Id welche zu selectieren ist
     */
    public void selectLog(String id){
        this.selectedLog = getLog(id);
    }

    /**
     * Holt das selectierte Log als Log Objekt
     * @return Das selectierte Log Objekt
     */
    public Log getSelectedLog(){
        return this.selectedLog;
    }

    /**
     * Holt ein Log anhand seiner ID
     * @param id Id des Log Eintrags
     * @return Das gefundene Log
     */
    public Log getLog(String id){
        AtomicReference<Log> returnLog = new AtomicReference<>();
        this.logData.forEach(s -> {
            if(s.getId().equals(id)){
                returnLog.set(s);
            }
        });
        return returnLog.get();
    }

    public ObservableList<Log> getLogData() {
        return logData;
    }

    public void setSelectedTour(Tour selectedTour) {
        this.selectedTour = selectedTour;
    }

    /**
     * Holt das Tourobjekt anhand des Namens
     * @param tourname Name der Tour
     * @return Gefundene Tour
     */
    public Tour getTour(String tourname){
        AtomicReference<Tour> returnTour = new AtomicReference<>();
        tourData.forEach(s -> {
            if(s.getName().equals(tourname)){
                returnTour.set(s);
            }
        });
        return returnTour.get();
    }

    /**
     * Selectiert eine Tour anhand des eindeutigen Namens
     * @param tourname Der Name der Tour
     */
    public void selectTour(String tourname){
        this.selectedTour = getTour(tourname);
        this.logData.removeIf(s -> true); // Löscht alles aus dem Array
        this.logData.addAll(this.selectedTour.getLogs());
    }

//    /**
//     * Selectiert eine Tour anhand des Tour Objects.
//     * Kann z.b. verwerndet werden um das selectierte Tour Object zu bearbeiten
//     * @param selected
//     */
//    public void setSelectedTour(Tour selected){
//        this.selectedTour = selected;
//    }

    public Tour getSelectedTour() {
        return selectedTour;
    }

    public String getNeueTourZiel() {
        return neueTourZiel;
    }

    public void setNeueTourZiel(String neueTourZiel) {
        this.neueTourZiel = neueTourZiel;
    }

    public ObservableList<String> getTourNamen() {
        return tourNamen;
    }

    public String getNeueTourName() {
        return neueTourName;
    }

    public void setNeueTourName(String neueTourName) {
        this.neueTourName = neueTourName;
    }

    public String getNeueTourStart() {
        return neueTourStart;
    }

    public void setNeueTourStart(String neueTourStart) {
        this.neueTourStart = neueTourStart;
    }

    public ObservableList<Tour> getTourData() {
        return tourData;
    }

    /**
     * Entfernt eine Tour anhand der ausgewählten Tour
     */
    public void delTour(){
        try {
            String tourname = this.selectedTour.getName();
            tourData.removeIf(s -> s.getName().equals(tourname));
            tourNamen.removeIf(s -> s.equals(tourname));
            logData.removeIf(s -> true);
        }catch (NullPointerException e){
            LogHelper.error(ConfigHelper.getLangIniString("keinetourselected"),
                    ConfigHelper.getLangIniString("deltournoselect"));
            AlertHelper.warn(ConfigHelper.getLangIniString("achtung"),
                    ConfigHelper.getLangIniString("keinetourselected"),
                    ConfigHelper.getLangIniString("deltournoselectmsg"));
        }
    }

    /**
     * Sucht eine Tour
     */
    public void suche(String suchString){

    }
}
